@@ -1,3 +1,10 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2018, Anaconda, Inc. All rights reserved.
+#
+# Powered by the Bokeh Development Team.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Provide Python descriptors for delegating to Bokeh properties.
 
 The Python `descriptor protocol`_ allows fine-grained control over all
@@ -37,7 +44,7 @@ set operations to the ``Float`` property.
 
     # But can raise a validation exception if an attempt to set to a list
     # is made
-    rnd.end = [1,2,3]   # ValueError !
+    rng.end = [1,2,3]   # ValueError !
 
 More sophisticated properties such as ``DataSpec`` and its subclasses can
 exert control over how values are serialized. Consider this example with
@@ -69,13 +76,48 @@ that can be used to attach Bokeh properties to Bokeh models.
 .. _descriptor protocol: https://docs.python.org/3/howto/descriptor.html
 
 '''
-from __future__ import absolute_import
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+
+# Standard library imports
 from copy import copy
 
+# External imports
 from six import string_types
 
-from .containers import PropertyValueContainer
+# Bokeh imports
+from .wrappers import PropertyValueContainer
+
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'BasicPropertyDescriptor',
+    'ColumnDataPropertyDescriptor',
+    'DataSpecPropertyDescriptor',
+    'PropertyDescriptor',
+    'UnitsSpecPropertyDescriptor',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
 
 class PropertyDescriptor(object):
     ''' Base class for a python descriptor that delegates access for a named
@@ -208,7 +250,8 @@ class PropertyDescriptor(object):
             None
 
         '''
-        from ..properties import DataSpec, ContainerProperty
+        from .bases import ContainerProperty
+        from .dataspec import DataSpec
         name = self.name
         if name in new_class_attrs:
             raise RuntimeError("Two property generators both created %s.%s" % (class_name, name))
@@ -253,7 +296,7 @@ class PropertyDescriptor(object):
         value = self.__get__(obj, obj.__class__)
         return self.property.serialize_value(value)
 
-    def set_from_json(self, obj, json, models, setter=None):
+    def set_from_json(self, obj, json, models=None, setter=None):
         '''Sets the value of this property from a JSON value.
 
         Args:
@@ -282,7 +325,7 @@ class PropertyDescriptor(object):
             None
 
         '''
-        self._internal_set(obj, json, setter)
+        self._internal_set(obj, json, setter=setter)
 
     def trigger_if_changed(self, obj, old):
         ''' Send a change event notification if the property is set to a
@@ -354,7 +397,7 @@ class PropertyDescriptor(object):
         '''
         raise NotImplementedError("Implement serialized()")
 
-    def _internal_set(self, obj, value, setter=None):
+    def _internal_set(self, obj, value, hint=None, setter=None):
         ''' Internal implementation to set property values, that is used
         by __set__, set_from_json, etc.
 
@@ -365,6 +408,14 @@ class PropertyDescriptor(object):
             old (obj) :
                 The previous value of the property to compare
 
+            hint (event hint or None, optional)
+                An optional update event hint, e.g. ``ColumnStreamedEvent``
+                (default: None)
+
+                Update event hints are usually used at times when better
+                update performance can be obtained by special-casing in
+                some way (e.g. streaming or patching column data sources)
+
             setter (ClientSession or ServerSession or None, optional) :
                 This is used to prevent "boomerang" updates to Bokeh apps.
                 (default: None)
@@ -492,7 +543,7 @@ class BasicPropertyDescriptor(PropertyDescriptor):
         if self.property._readonly:
             raise RuntimeError("%s.%s is a readonly property" % (obj.__class__.__name__, self.name))
 
-        self._internal_set(obj, value, setter)
+        self._internal_set(obj, value, setter=setter)
 
     def __delete__(self, obj):
         ''' Implement the deleter for the Python `descriptor protocol`_.
@@ -544,7 +595,7 @@ class BasicPropertyDescriptor(PropertyDescriptor):
         Args:
             obj (HasProps) :
 
-            json (JSON) :
+            json (JSON-dict) :
 
             models(seq[Model], optional) :
 
@@ -662,26 +713,21 @@ class BasicPropertyDescriptor(PropertyDescriptor):
         default = self.instance_default(obj)
 
         if is_themed:
-            if self.name in obj._unstable_themed_values:
-                return obj._unstable_themed_values[self.name]
+            unstable_dict = obj._unstable_themed_values
+        else:
+            unstable_dict = obj._unstable_default_values
 
-            if self.property._may_have_unstable_default():
-                if isinstance(default, PropertyValueContainer):
-                    default._register_owner(obj, self)
-                obj._unstable_themed_values[self.name] = default
-            return default
-
-        if self.name in obj._unstable_default_values:
-            return obj._unstable_default_values[self.name]
+        if self.name in unstable_dict:
+            return unstable_dict[self.name]
 
         if self.property._may_have_unstable_default():
             if isinstance(default, PropertyValueContainer):
                 default._register_owner(obj, self)
-            obj._unstable_default_values[self.name] = default
+            unstable_dict[self.name] = default
 
         return default
 
-    def _internal_set(self, obj, value, setter=None):
+    def _internal_set(self, obj, value, hint=None, setter=None):
         ''' Internal implementation to set property values, that is used
         by __set__, set_from_json, etc.
 
@@ -695,6 +741,14 @@ class BasicPropertyDescriptor(PropertyDescriptor):
             old (obj) :
                 The previous value of the property to compare
 
+            hint (event hint or None, optional)
+                An optional update event hint, e.g. ``ColumnStreamedEvent``
+                (default: None)
+
+                Update event hints are usually used at times when better
+                update performance can be obtained by special-casing in
+                some way (e.g. streaming or patching column data sources)
+
             setter (ClientSession or ServerSession or None, optional) :
                 This is used to prevent "boomerang" updates to Bokeh apps.
                 (default: None)
@@ -713,14 +767,13 @@ class BasicPropertyDescriptor(PropertyDescriptor):
         value = self.property.prepare_value(obj, self.name, value)
 
         old = self.__get__(obj, obj.__class__)
-        self._real_set(obj, old, value, setter=setter)
+        self._real_set(obj, old, value, hint=hint, setter=setter)
 
     def _real_set(self, obj, old, value, hint=None, setter=None):
         ''' Internal implementation helper to set property values.
 
         This function handles bookkeeping around noting whether values have
-        been explicitly set, or setting up |PropertyContainer| wrappers when
-        values are containers, etc.
+        been explicitly set, etc.
 
         Args:
             obj (HasProps)
@@ -729,6 +782,14 @@ class BasicPropertyDescriptor(PropertyDescriptor):
             old (obj) :
                 The previous value of the property to compare
 
+            hint (event hint or None, optional)
+                An optional update event hint, e.g. ``ColumnStreamedEvent``
+                (default: None)
+
+                Update event hints are usually used at times when better
+                update performance can be obtained by special-casing in
+                some way (e.g. streaming or patching column data sources)
+
             setter (ClientSession or ServerSession or None, optional) :
                 This is used to prevent "boomerang" updates to Bokeh apps.
                 (default: None)
@@ -744,17 +805,17 @@ class BasicPropertyDescriptor(PropertyDescriptor):
             None
 
         '''
-        # As of Bokeh 0.11.1, all hinted events modify in place. However this
-        # may need refining later if this assumption changes.
-        unchanged = self.property.matches(value, old) and (hint is None)
-        if unchanged:
+        # Normally we want a "no-op" if the new value and old value are identical
+        # but some hinted events are in-place. This check will allow those cases
+        # to continue on to the notification machinery
+        if self.property.matches(value, old) and (hint is None):
             return
 
         was_set = self.name in obj._property_values
 
-        # "old" is the logical old value, but it may not be
-        # the actual current attribute value if our value
-        # was mutated behind our back and we got _notify_mutated.
+        # "old" is the logical old value, but it may not be the actual current
+        # attribute value if our value was mutated behind our back and we got
+        # _notify_mutated.
         if was_set:
             old_attr_value = obj._property_values[self.name]
         else:
@@ -775,7 +836,7 @@ class BasicPropertyDescriptor(PropertyDescriptor):
             obj._property_values[self.name] = value
 
         # for notification purposes, "old" should be the logical old
-        self._trigger(obj, old, value, hint, setter)
+        self._trigger(obj, old, value, hint=hint, setter=setter)
 
     # called when a container is mutated "behind our back" and
     # we detect it with our collection wrappers.
@@ -812,7 +873,7 @@ class BasicPropertyDescriptor(PropertyDescriptor):
         # in some cases this could give us a new object for the value
         value = self.property.prepare_value(obj, self.name, value)
 
-        self._real_set(obj, old, value, hint)
+        self._real_set(obj, old, value, hint=hint)
 
     def _trigger(self, obj, old, value, hint=None, setter=None):
         ''' Unconditionally send a change event notification for the property.
@@ -855,6 +916,63 @@ class BasicPropertyDescriptor(PropertyDescriptor):
             obj.trigger(self.name, old, value, hint, setter)
 
 
+class ColumnDataPropertyDescriptor(BasicPropertyDescriptor):
+    ''' A PropertyDescriptor specialized to handling ``ColumnData`` properties.
+
+    '''
+
+    def __set__(self, obj, value, setter=None):
+        ''' Implement the setter for the Python `descriptor protocol`_.
+
+        This method first separately extracts and removes any ``units`` field
+        in the JSON, and sets the associated units property directly. The
+        remaining value is then passed to the superclass ``__set__`` to
+        be handled.
+
+        .. note::
+            An optional argument ``setter`` has been added to the standard
+            setter arguments. When needed, this value should be provided by
+            explicitly invoking ``__set__``. See below for more information.
+
+        Args:
+            obj (HasProps) :
+                The instance to set a new property value on
+
+            value (obj) :
+                The new value to set the property to
+
+            setter (ClientSession or ServerSession or None, optional) :
+                This is used to prevent "boomerang" updates to Bokeh apps.
+                (default: None)
+
+                In the context of a Bokeh server application, incoming updates
+                to properties will be annotated with the session that is
+                doing the updating. This value is propagated through any
+                subsequent change notifications that the update triggers.
+                The session can compare the event setter to itself, and
+                suppress any updates that originate from itself.
+
+        Returns:
+            None
+
+        '''
+        if not hasattr(obj, '_property_values'):
+            # Initial values should be passed in to __init__, not set directly
+            raise RuntimeError("Cannot set a property value '%s' on a %s instance before HasProps.__init__" %
+                               (self.name, obj.__class__.__name__))
+
+        if self.property._readonly:
+            raise RuntimeError("%s.%s is a readonly property" % (obj.__class__.__name__, self.name))
+
+        from ...document.events import ColumnDataChangedEvent
+
+        if obj.document:
+            hint = ColumnDataChangedEvent(obj.document, obj, setter=setter)
+        else:
+            hint = None
+
+        self._internal_set(obj, value, hint=hint, setter=setter)
+
 class DataSpecPropertyDescriptor(BasicPropertyDescriptor):
     ''' A PropertyDescriptor for Bokeh |DataSpec| properties that serialize to
     field/value dictionaries.
@@ -875,7 +993,7 @@ class DataSpecPropertyDescriptor(BasicPropertyDescriptor):
         Args:
             obj (HasProps) :
 
-            json (JSON) :
+            json (JSON-dict) :
 
             models(seq[Model], optional) :
 
@@ -900,7 +1018,7 @@ class DataSpecPropertyDescriptor(BasicPropertyDescriptor):
             old = getattr(obj, self.name)
             if old is not None:
                 try:
-                    self.property._type.validate(old)
+                    self.property._type.validate(old, False)
                     if 'value' in json:
                         json = json['value']
                 except ValueError:
@@ -1031,3 +1149,11 @@ class UnitsSpecPropertyDescriptor(DataSpecPropertyDescriptor):
             if units:
                 self.units_prop.__set__(obj, units)
         return value
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
