@@ -1,9 +1,36 @@
-import pytest
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2018, Anaconda, Inc. All rights reserved.
+#
+# Powered by the Bokeh Development Team.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import pytest ; pytest
+
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
 from mock import patch
 
+# Module under test
 import bokeh.document.events as bde
 
+#-----------------------------------------------------------------------------
+# Setup
+#-----------------------------------------------------------------------------
 
 class FakeEmptyDispatcher(object): pass
 
@@ -25,9 +52,13 @@ class FakeModel(object):
     data = "data"
     def references(self): return dict(ref1=1, ref2=2)
 
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 # DocumentChangedEvent --------------------------------------------------------
 
-class TesDocumentChangedEvent(object):
+class TestDocumentChangedEvent(object):
 
     def test_init(self):
         e = bde.DocumentChangedEvent("doc")
@@ -269,6 +300,13 @@ class TestColumnsStreamedEvent(object):
         assert e.data == dict(foo=1)
         assert e.rollover == 200
 
+    def test_pandas_data(self, pd):
+        m = FakeModel()
+        df = pd.DataFrame({'x': [1, 2, 3], 'y': [4, 5, 6]})
+        e = bde.ColumnsStreamedEvent("doc", m, df, 200, "setter", "invoker")
+        assert isinstance(e.data, dict)
+        assert e.data == {c: df[c] for c in df.columns}
+
 # ColumnsPatchedEvent ---------------------------------------------------------
 
 class TestColumnsPatchedEvent(object):
@@ -467,3 +505,15 @@ class TestSessionCallbackRemoved(object):
         e = bde.SessionCallbackAdded("doc", "setter")
         e2 = bde.SessionCallbackAdded("doc", "setter")
         assert e.combine(e2) == False
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
