@@ -6,10 +6,12 @@ from ..core.has_props import abstract
 from ..core.properties import Any, Bool, ColumnData, Dict, Enum, Instance, Int, JSON, List, Seq, String
 from ..model import Model
 from ..util.dependencies import import_optional
+from ..util.serialization import convert_datetime_array
 from ..util.warnings import BokehUserWarning
 
-from .callbacks import Callback
+from .callbacks import Callback, CustomJS
 from .filters import Filter
+from .selections import Selection, SelectionPolicy, UnionRenderers
 
 pd = import_optional('pandas')
 
@@ -19,41 +21,16 @@ class DataSource(Model):
 
     '''
 
-    selected = Dict(String, Dict(String, Any), default={
-        '0d': {'glyph': None, 'indices': []},
-        '1d': {'indices': []},
-        '2d': {'indices': {}}
-    }, help="""
-    A dict to indicate selected indices on different dimensions on this DataSource. Keys are:
-
-    .. code-block:: python
-
-        # selection information for line and patch glyphs
-        '0d' : {
-          # the glyph that was selected
-          'glyph': None
-
-          # array with the [smallest] index of the segment of the line that was hit
-          'indices': []
-        }
-
-        # selection for most (point-like) glyphs, except lines and patches
-        '1d': {
-          # indices of the points included in the selection
-          indices: []
-        }
-
-        # selection information for multiline and patches glyphs
-        '2d': {
-          # mapping of indices of the multiglyph to array of glyph indices that were hit
-          # e.g. {3: [5, 6], 4: [5]}
-          indices: {}
-        }
-
+    selected = Instance(Selection, default=lambda: Selection(), help="""
+    A Selection that indicates selected indices on this DataSource.
     """)
 
     callback = Instance(Callback, help="""
     A callback to run in the browser whenever the selection is changed.
+
+    .. note:
+        This property is left for backwards compatibility, but may be deprecated
+        in the future. Prefer ``source.selected.js_on_change(...)`` for new code.
     """)
 
 @abstract
@@ -63,8 +40,8 @@ class ColumnarDataSource(DataSource):
 
     '''
 
-    column_names = List(String, help="""
-    An list of names for all the columns in this DataSource.
+    selection_policy = Instance(SelectionPolicy, default=lambda: UnionRenderers(), help="""
+    An instance of a SelectionPolicy that determines how selections are set.
     """)
 
 class ColumnDataSource(ColumnarDataSource):
@@ -85,6 +62,11 @@ class ColumnDataSource(ColumnarDataSource):
 
           source = ColumnDataSource(data)
 
+    .. note::
+        ``ColumnDataSource`` only creates a shallow copy of ``data``. Use e.g.
+        ``ColumnDataSource(copy.deepcopy(data))`` if initializing from another
+        ``ColumnDataSource.data`` object that you want to keep independent.
+
     * A Pandas ``DataFrame`` object
 
       .. code-block:: python
@@ -92,10 +74,15 @@ class ColumnDataSource(ColumnarDataSource):
           source = ColumnDataSource(df)
 
       In this case the CDS will have columns corresponding to the columns of
-      the ``DataFrame``. If the ``DataFrame`` has a named index column, then
-      CDS will also have a column with this name. However, if the index name
-      (or any subname of a ``MultiIndex``) is ``None``, then the CDS will have
-      a column generically named ``index`` for the index.
+      the ``DataFrame``. If the ``DataFrame`` columns have multiple levels,
+      they will be flattend using an underscore (e.g. level_0_col_level_1_col).
+      The index of the DataFrame will be flattened to an ``Index`` of tuples
+      if it's a ``MultiIndex``, and then reset using ``reset_index``. The result
+      will be a column with the same name if the index was named, or
+      level_0_name_level_1_name if it was a named ``MultiIndex``. If the
+      ``Index`` did not have a name or the ``MultiIndex`` name could not be
+      flattened/determined, the ``reset_index`` function will name the index column
+      ``index``, or ``level_0`` if the name ``index`` is not available.
 
     * A Pandas ``GroupBy`` object
 
@@ -155,9 +142,15 @@ class ColumnDataSource(ColumnarDataSource):
             else:
                 raise ValueError("expected a dict or pandas.DataFrame, got %s" % raw_data)
         super(ColumnDataSource, self).__init__(**kw)
-        self.column_names[:] = list(raw_data.keys())
         self.data.update(raw_data)
 
+    @property
+    def column_names(self):
+        ''' A list of the column names in this data source.
+
+        '''
+        return list(self.data)
+
     @staticmethod
     def _data_from_df(df):
         ''' Create a ``dict`` of columns from a Pandas DataFrame,
@@ -171,24 +164,28 @@ class ColumnDataSource(ColumnarDataSource):
 
         '''
         _df = df.copy()
-        index = _df.index
+
+        # Flatten columns
+        if isinstance(df.columns, pd.MultiIndex):
+            try:
+                _df.columns = ['_'.join(col) for col in _df.columns.values]
+            except TypeError:
+                raise TypeError('Could not flatten MultiIndex columns. '
+                                'use string column names or flatten manually')
+        # Flatten index
+        index_name = ColumnDataSource._df_index_name(df)
+        if index_name == 'index':
+            _df.index = pd.Index(_df.index.values)
+        else:
+            _df.index = pd.Index(_df.index.values, name=index_name)
+        _df.reset_index(inplace=True)
+
         tmp_data = {c: v.values for c, v in _df.iteritems()}
 
         new_data = {}
         for k, v in tmp_data.items():
-            if isinstance(k, tuple):
-                k = "_".join(k)
             new_data[k] = v
 
-        if index.name:
-            new_data[index.name] = index.values
-        elif index.names:
-            try:
-                new_data["_".join(index.names)] = index.values
-            except TypeError:
-                new_data["index"] = index.values
-        else:
-            new_data["index"] = index.values
         return new_data
 
     @staticmethod
@@ -208,6 +205,38 @@ class ColumnDataSource(ColumnarDataSource):
         '''
         return ColumnDataSource._data_from_df(group.describe())
 
+    @staticmethod
+    def _df_index_name(df):
+        ''' Return the Bokeh-appropriate column name for a DataFrame index
+
+        If there is no named index, then `"index" is returned.
+
+        If there is a single named index, then ``df.index.name`` is returned.
+
+        If there is a multi-index, and the index names are all strings, then
+        the names are joined with '_' and the result is returned, e.g. for a
+        multi-index ``['ind1', 'ind2']`` the result will be "ind1_ind2".
+        Otherwise if any index name is not a string, the fallback name "index"
+        is returned.
+
+        Args:
+            df (DataFrame) : the DataFrame to find an index name for
+
+        Returns:
+            str
+
+        '''
+        if df.index.name:
+            return df.index.name
+        elif df.index.names:
+            try:
+                return "_".join(df.index.names)
+            except TypeError:
+                return "index"
+        else:
+            return "index"
+
+
     @classmethod
     def from_df(cls, data):
         ''' Create a ``dict`` of columns from a Pandas DataFrame,
@@ -242,20 +271,13 @@ class ColumnDataSource(ColumnarDataSource):
     def to_df(self):
         ''' Convert this data source to pandas dataframe.
 
-        If ``column_names`` is set, use those. Otherwise let Pandas
-        infer the column names. The ``column_names`` property can be
-        used both to order and filter the columns.
-
         Returns:
             DataFrame
 
         '''
         if not pd:
             raise RuntimeError('Pandas must be installed to convert to a Pandas Dataframe')
-        if self.column_names:
-            return pd.DataFrame(self.data, columns=self.column_names)
-        else:
-            return pd.DataFrame(self.data)
+        return pd.DataFrame(self.data)
 
     def add(self, data, name=None):
         ''' Appends a new column of data to the data source.
@@ -274,7 +296,6 @@ class ColumnDataSource(ColumnarDataSource):
             while "Series %d"%n in self.data:
                 n += 1
             name = "Series %d"%n
-        self.column_names.append(name)
         self.data[name] = data
         return name
 
@@ -293,7 +314,6 @@ class ColumnDataSource(ColumnarDataSource):
 
         '''
         try:
-            self.column_names.remove(name)
             del self.data[name]
         except (ValueError, KeyError):
             import warnings
@@ -392,13 +412,24 @@ class ColumnDataSource(ColumnarDataSource):
             source.stream(new_data)
 
         '''
+        needs_length_check = True
+
         if pd and isinstance(new_data, pd.Series):
             new_data = new_data.to_frame().T
+
         if pd and isinstance(new_data, pd.DataFrame):
-            newkeys = set(new_data.columns)
+            needs_length_check = False # DataFrame lengths equal by definition
+            _df = new_data
+            newkeys = set(_df.columns)
+            index_name = ColumnDataSource._df_index_name(_df)
+            newkeys.add(index_name)
+            new_data = dict(_df.iteritems())
+            new_data[index_name] = _df.index.values
         else:
             newkeys = set(new_data.keys())
+
         oldkeys = set(self.data.keys())
+
         if newkeys != oldkeys:
             missing = oldkeys - newkeys
             extra = newkeys - oldkeys
@@ -411,9 +442,8 @@ class ColumnDataSource(ColumnarDataSource):
             else:
                 raise ValueError("Must stream updates to all existing columns (extra: %s)" % ", ".join(sorted(extra)))
 
-        if not (pd and isinstance(new_data, pd.DataFrame)):
-            import numpy as np
-
+        import numpy as np
+        if needs_length_check:
             lengths = set()
             arr_types = (np.ndarray, pd.Series) if pd else np.ndarray
             for k, x in new_data.items():
@@ -427,6 +457,20 @@ class ColumnDataSource(ColumnarDataSource):
             if len(lengths) > 1:
                 raise ValueError("All streaming column updates must be the same length")
 
+        # slightly awkward that we have to call convert_datetime_array here ourselves
+        # but the downstream code expects things to already be ms-since-epoch
+        for key, values in new_data.items():
+            if pd and isinstance(values, (pd.Series, pd.Index)):
+                values = values.values
+            old_values = self.data[key]
+            # Apply the transformation if the new data contains datetimes
+            # but the current data has already been transformed
+            if (isinstance(values, np.ndarray) and values.dtype.kind.lower() == 'm' and
+                isinstance(old_values, np.ndarray) and old_values.dtype.kind.lower() != 'm'):
+                new_data[key] = convert_datetime_array(values)
+            else:
+                new_data[key] = values
+
         self.data._stream(self.document, self, new_data, rollover, setter)
 
     def patch(self, patches, setter=None):
@@ -620,37 +664,77 @@ class GeoJSONDataSource(ColumnarDataSource):
 
 @abstract
 class RemoteSource(ColumnDataSource):
-    '''
+    ''' Base class for remote column data sources that can update from data
+    URLs at prescribed time intervals.
+
+    .. note::
+        This base class is typically not useful to instantiate on its own.
 
     '''
 
     data_url = String(help="""
-    The URL to the endpoint for the data.
+    A URL to to fetch data from.
     """)
 
     polling_interval = Int(help="""
-    polling interval for updating data source in milliseconds
+    A polling interval (in milliseconds) for updating data source.
     """)
 
 class AjaxDataSource(RemoteSource):
-    '''
+    ''' A data source that can populate columns by making Ajax calls to REST
+    enpoints.
+
+    The ``AjaxDataSource`` can be especially useful if you want to make a
+    standalone document (i.e. not backed by the Bokeh server) that can still
+    dynamically update using an existing REST API.
+
+    The response from the REST API should match the ``.data`` property of a
+    standard ``ColumnDataSource``, i.e. a JSON dict that maps names to arrays
+    of values:
+
+    .. code-block:: python
+
+        {
+            'x' : [1, 2, 3, ...],
+            'y' : [9, 3, 2, ...]
+        }
+
+    Alternatively, if the REST API returns a different format, a CustomJS
+    callback can be provided to convert the REST response into Bokeh format,
+    via the ``adapter`` property of this data source.
+
+    A full example can be seen at :bokeh-tree:`examples/howto/ajax_source.py`
 
     '''
 
-    method = Enum('POST', 'GET', help="http method - GET or POST")
+    method = Enum('POST', 'GET', help="""
+    Specifiy the the HTTP method to use for the Ajax request (GET or POST)
+    """)
 
     mode = Enum("replace", "append", help="""
-    Whether to append new data to existing data (up to ``max_size``),
-    or to replace existing data entirely.
+    Whether to append new data to existing data (up to ``max_size``), or to
+    replace existing data entirely.
+    """)
+
+    adapter = Instance(CustomJS, help="""
+    A JavaScript callback to adapt raw JSON responses to Bokeh ColumnDataSource
+    format.
+
+    If provided, this callback is executes immediately after the JSON data is
+    received, but before apprending or replacing data in the data source. The
+    CustomJS callback will receive the AjaxDataSource as ``cb_obj`` and will
+    receive the raw JSON response as ``cb_data.response``. The callback code
+    should return a ``data`` object suitable for a Bokeh ColumnDataSource (i.e.
+    a mapping of string column names to arrays of data).
     """)
 
     max_size = Int(help="""
-    Maximum size of the data array being kept after each pull requests.
-    Larger than that size, the data will be right shifted.
+    Maximum size of the data columns. If a new fetch would result in columns
+    larger than ``max_size``, then earlier data is dropped to make room.
     """)
 
     if_modified = Bool(False, help="""
-    Whether to include an ``If-Modified-Since`` header in AJAX requests
+    Whether to include an ``If-Modified-Since`` header in Ajax requests
     to the server. If this header is supported by the server, then only
     new data since the last request will be returned.
     """)
@@ -660,5 +744,12 @@ class AjaxDataSource(RemoteSource):
     """)
 
     http_headers = Dict(String, String, help="""
-    HTTP headers to set for the Ajax request.
+    Specify HTTP headers to set for the Ajax request.
+
+    Example:
+
+    .. code-block:: python
+
+        ajax_source.headers = { 'x-my-custom-header': 'some value' }
+
     """)
