@@ -28,7 +28,7 @@ from ..core.enums import (Anchor, Dimension, Dimensions, Location,
                           TooltipFieldFormatter, TooltipAttachment)
 from ..core.has_props import abstract
 from ..core.properties import (
-    Auto, Bool, Color, Date, Datetime, Dict, Either, Enum, Int, Float,
+    Auto, Bool, Color, Date, Datetime, Dict, Either, Enum, Image, Int, Float,
     Percent, Instance, List, Seq, String, Tuple
 )
 from ..util.compiler import nodejs_compile, CompilationError
@@ -38,13 +38,14 @@ from ..core.validation import error
 from ..core.validation.errors import (
     INCOMPATIBLE_BOX_EDIT_RENDERER, INCOMPATIBLE_POINT_DRAW_RENDERER,
     INCOMPATIBLE_POLY_DRAW_RENDERER, INCOMPATIBLE_POLY_EDIT_RENDERER,
-    INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER
+    INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER, NO_RANGE_TOOL_RANGES
 )
 from ..model import Model
 
 from .annotations import BoxAnnotation, PolyAnnotation
 from .callbacks import Callback
 from .glyphs import XYGlyph, Rect, Patches, MultiLine
+from .ranges import Range1d
 from .renderers import Renderer, GlyphRenderer
 from .layouts import LayoutDOM
 
@@ -62,28 +63,35 @@ class Action(Tool):
     pass
 
 @abstract
-class Drag(Tool):
+class Gesture(Tool):
     ''' A base class for tools that respond to drag events.
 
     '''
     pass
 
 @abstract
-class Scroll(Tool):
+class Drag(Gesture):
+    ''' A base class for tools that respond to drag events.
+
+    '''
+    pass
+
+@abstract
+class Scroll(Gesture):
     ''' A base class for tools that respond to scroll events.
 
     '''
     pass
 
 @abstract
-class Tap(Tool):
+class Tap(Gesture):
     ''' A base class for tools that respond to tap/click events.
 
     '''
     pass
 
 @abstract
-class Inspection(Tool):
+class Inspection(Gesture):
     ''' A base class for tools that perform "inspections", e.g. ``HoverTool``.
 
     '''
@@ -104,6 +112,11 @@ class ToolbarBase(Model):
     set to None, no logo will be displayed.
     """)
 
+    autohide = Bool(default=False, help="""
+    Whether the toolbar will be hidden by default. Default: False.
+    If True, hides toolbar when cursor is not in canvas.
+    """)
+
     tools = List(Instance(Tool), help="""
     A list of tools to add to the plot.
     """)
@@ -130,6 +143,16 @@ class Toolbar(ToolbarBase):
     Specify a tap/click tool to be active when the plot is displayed.
     """)
 
+    active_multi = Instance((Gesture), help="""
+    Specify an active multi-gesture tool, for instance an edit tool or a range
+    tool.
+
+    Note that activating a multi-gesture tool will deactivate any other gesture
+    tools as appropriate. For example, if a pan tool is set as the active drag,
+    and this property is set to a ``BoxEditTool`` instance, the pan tool will
+    be deactivated (i.e. the multi-gesture tool will take precedence).
+    """)
+
 class ProxyToolbar(ToolbarBase):
     ''' A toolbar that allow to merge and proxy tools of toolbars in multiple plots. '''
 
@@ -169,6 +192,73 @@ class PanTool(Drag):
     height of the plot.
     """)
 
+DEFAULT_RANGE_OVERLAY = lambda: BoxAnnotation(
+    level="overlay",
+    render_mode="css",
+    fill_color="lightgrey",
+    fill_alpha=0.5,
+    line_color="black",
+    line_alpha=1.0,
+    line_width=0.5,
+    line_dash=[2,2],
+)
+
+class RangeTool(Drag):
+    ''' *toolbar icon*: |range_icon|
+
+    The range tool allows the user to update range objects for either or both
+    of the x- or y-dimensions by dragging a corresponding shaded annotation to
+    move it or change its boundaries.
+
+    A common use case is to add this tool to a plot with a large fixed range,
+    but to configure the tool range from a different plot. When the user
+    manipulates the overlay, the range of the second plot will be updated
+    automatically.
+
+    .. |range_icon| image:: /_images/icons/Range.png
+        :height: 18pt
+
+    '''
+
+    x_range = Instance(Range1d, help="""
+    A range synchronized to the x-dimension of the overlay. If None, the overlay
+    will span the entire x-dimension.
+    """)
+
+    x_interaction = Bool(default=True, help="""
+    Whether to respond to horizontal pan motions when an ``x_range`` is present.
+
+    By default, when an ``x_range`` is specified, it is possible to adjust the
+    horizontal position of the range box by panning horizontally inside the
+    box, or along the top or bottom edge of the box. To disable this, and fix
+    the  range box in place horizontally, set to False. (The box will still
+    update if the ``x_range`` is updated programmatically.)
+    """)
+
+    y_range = Instance(Range1d, help="""
+    A range synchronized to the y-dimension of the overlay. If None, the overlay
+    will span the entire y-dimension.
+    """)
+
+    y_interaction = Bool(default=True, help="""
+    Whether to respond to vertical pan motions when a ``y_range`` is present.
+
+    By default, when a ``y_range`` is specified, it is possible to adjust the
+    vertical position of the range box by panning vertically inside the box, or
+    along the top or bottom edge of the box. To disable this, and fix the range
+    box in place vertically, set to False. (The box will still update if the
+    ``y_range`` is updated programmatically.)
+    """)
+
+    overlay = Instance(BoxAnnotation, default=DEFAULT_RANGE_OVERLAY, help="""
+    A shaded annotation drawn to indicate the configured ranges.
+    """)
+
+    @error(NO_RANGE_TOOL_RANGES)
+    def _check_no_range_tool_ranges(self):
+        if self.x_range is None and self.y_range is None:
+            return "At least one of RangeTool.x_range or RangeTool.y_range must be configured"
+
 class WheelPanTool(Scroll):
     ''' *toolbar icon*: |wheel_pan_icon|
 
@@ -224,6 +314,36 @@ class WheelZoomTool(Scroll):
     0.001 and 0.09. High values will be clipped. Speed may very between browsers.
     """)
 
+class CustomAction(Action):
+    ''' Execute a custom action, e.g. ``CustomJS`` callback when a toolbar
+    icon is activated.
+
+    Example:
+
+        .. code-block:: python
+
+            tool = CustomAction(icon="icon.png",
+                                callback=CustomJS(code='alert("foo")'))
+
+            plot.add_tools(tool)
+
+    '''
+
+    action_tooltip = String(default="Perform a Custom Action", help="""
+    Tooltip displayed when hovering over the custom action icon.
+    """)
+
+    callback = Instance(Callback, help="""
+    A Bokeh callback to execute when the custom action icon is activated.
+    """)
+
+    icon = Image(help="""
+    An icon to display in the toolbar.
+
+    The icon can provided as a string filename for an image, a PIL ``Image``
+    object, or an RGB(A) NumPy array.
+    """)
+
 class SaveTool(Action):
     ''' *toolbar icon*: |save_icon|
 
@@ -309,6 +429,13 @@ class TapTool(Tap):
     its ``.source`` attribute and the selection geometry as its
     ``.geometries`` attribute.
 
+    The ``.geometries`` attribute has 5 members.
+    ``.type`` is the geometry type, which always a ``.point`` for a tap event.
+    ``.sx`` and ``.sy`` are the screen X and Y coordinates where the tap occurred.
+    ``.x`` and ``.y`` are the converted data coordinates for the item that has
+    been selected. The ``.x`` and ``.y`` values are based on the axis assiged
+    to that glyph.
+
     .. note::
         This callback does *not* execute on every tap, only when a glyphs is
         "hit". If you would like to execute a callback on every mouse tap,
@@ -647,6 +774,7 @@ class CustomJSHover(Model):
     * ``data_x`` data-space x-coordinate of the hovered glyph
     * ``data_y`` data-space y-coordinate of the hovered glyph
     * ``indices`` column indices of all currently hovered glyphs
+    * ``name`` value of the ``name`` property of the hovered glyph renderer
 
     If the hover is over a "multi" glyph such as ``Patches`` or ``MultiLine``
     then a ``segment_index`` key will also be present.
@@ -671,7 +799,7 @@ class CustomJSHover(Model):
 
             p.add_tools(HoverTool(
                 tooltips=[( 'lat','@y{custom}' )],
-                formatter=dict(y=lat_custom)
+                formatters=dict(y=lat_custom)
             ))
 
     .. warning::
@@ -717,7 +845,7 @@ class CustomJSHover(Model):
             if len(all_names) - len(default_values) != 0:
                 raise ValueError("Function may only contain keyword arguments.")
 
-            if default_values and not any([isinstance(value, Model) for value in default_values]):
+            if default_values and not any(isinstance(value, Model) for value in default_values):
                 raise ValueError("Default value must be a Bokeh Model.")
 
             func_kwargs = dict(zip(all_names, default_values))
@@ -895,7 +1023,8 @@ class HoverTool(Inspection):
 
     Field names starting with "$" are special, known fields:
 
-    :$index: index of selected point in the data source
+    :$index: index of hovered point in the data source
+    :$name: value of the ``name`` property of the hovered glyph renderer
     :$x: x-coordinate under the cursor in data space
     :$y: y-coordinate under the cursor in data space
     :$sx: x-coordinate under the cursor in screen (canvas) space
@@ -915,6 +1044,13 @@ class HoverTool(Inspection):
     surrounding it in curly braces, e.g. ``@{adjusted close}`` will display
     values from a column named ``"adjusted close"``.
 
+    Sometimes (especially with stacked charts) it is desirable to allow the
+    name of the column be specified indirectly. The field name ``@$name`` is
+    distinguished in that it will look up the ``name`` field on the hovered
+    glyph renderer, and use that value as the column name. For instance, if
+    a user hovers with the name ``"US East"``, then ``@$name`` is equivalent to
+    ``@{US East}``.
+
     By default, values for fields (e.g. ``@foo``) are displayed in a basic
     numeric format. However it is possible to control the formatting of values
     more precisely. Fields can be modified by appending a format specified to
@@ -1051,11 +1187,15 @@ class RedoTool(Action):
     '''
 
 @abstract
-class EditTool(Tool):
+class EditTool(Gesture):
     ''' A base class for all interactive draw tool types.
 
     '''
 
+    custom_tooltip = String(None, help="""
+    A custom tooltip label to override the default name.
+    """)
+
     empty_value = Either(Bool, Int, Float, Date, Datetime, Color, help="""
     Defines the value to insert on non-coordinate columns when a new
     glyph is inserted into the ColumnDataSource columns, e.g. when a
@@ -1064,6 +1204,13 @@ class EditTool(Tool):
     the color column will be filled with the defined empty value.
     """)
 
+    custom_icon = Image(help="""
+    An icon to display in the toolbar.
+
+    The icon can provided as a string filename for an image, a PIL ``Image``
+    object, or an RGB(A) NumPy array.
+    """)
+
     renderers = List(Instance(Renderer), help="""
     An explicit list of renderers corresponding to scatter glyphs
     that may be edited.
@@ -1120,6 +1267,13 @@ class BoxEditTool(EditTool, Drag, Tap):
     the vertical dimension can be controlled.
     """)
 
+    num_objects = Int(default=0, help="""
+    Defines a limit on the number of boxes that can be drawn. By
+    default there is no limit on the number of objects, but if enabled
+    the oldest drawn box will be dropped to make space for the new box
+    being added.
+    """)
+
     @error(INCOMPATIBLE_BOX_EDIT_RENDERER)
     def _check_compatible_renderers(self):
         incompatible_renderers = []
@@ -1127,7 +1281,7 @@ class BoxEditTool(EditTool, Drag, Tap):
             if not isinstance(renderer.glyph, Rect):
                 incompatible_renderers.append(renderer)
         if incompatible_renderers:
-            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            glyph_types = ', '.join(type(renderer.glyph).__name__ for renderer in incompatible_renderers)
             return "%s glyph type(s) found." % glyph_types
 
 class PointDrawTool(EditTool, Drag, Tap):
@@ -1171,6 +1325,13 @@ class PointDrawTool(EditTool, Drag, Tap):
     drag = Bool(default=True, help="""
     Enables dragging of existing points on pan events.""")
 
+    num_objects = Int(default=0, help="""
+    Defines a limit on the number of points that can be drawn. By
+    default there is no limit on the number of objects, but if enabled
+    the oldest drawn point will be dropped to make space for the new
+    point.
+    """)
+
     @error(INCOMPATIBLE_POINT_DRAW_RENDERER)
     def _check_compatible_renderers(self):
         incompatible_renderers = []
@@ -1178,7 +1339,7 @@ class PointDrawTool(EditTool, Drag, Tap):
             if not isinstance(renderer.glyph, XYGlyph):
                 incompatible_renderers.append(renderer)
         if incompatible_renderers:
-            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            glyph_types = ', '.join(type(renderer.glyph).__name__ for renderer in incompatible_renderers)
             return "%s glyph type(s) found." % glyph_types
 
 class PolyDrawTool(EditTool, Drag, Tap):
@@ -1195,6 +1356,11 @@ class PolyDrawTool(EditTool, Drag, Tap):
     additional columns in the data source will be padded with the
     declared ``empty_value``, when adding a new point.
 
+    If a ``vertex_renderer`` with an point-like glyph is supplied the
+    PolyDrawTool will use it to display the vertices of the
+    multi-lines/patches on all supplied renderers. This also enables
+    the ability to snap to existing vertices while drawing.
+
     The supported actions include:
 
     * Add patch/multi-line: Double tap to add the first vertex, then
@@ -1217,6 +1383,17 @@ class PolyDrawTool(EditTool, Drag, Tap):
     drag = Bool(default=True, help="""
     Enables dragging of existing patches and multi-lines on pan events.""")
 
+    num_objects = Int(default=0, help="""
+    Defines a limit on the number of patches or multi-lines that can
+    be drawn. By default there is no limit on the number of objects,
+    but if enabled the oldest drawn patch or multi-line will be
+    dropped to make space for the new patch or multi-line.
+    """)
+
+    vertex_renderer = Instance(GlyphRenderer, help="""
+    The renderer used to render the vertices of a selected line or
+    polygon.""")
+
     @error(INCOMPATIBLE_POLY_DRAW_RENDERER)
     def _check_compatible_renderers(self):
         incompatible_renderers = []
@@ -1224,7 +1401,56 @@ class PolyDrawTool(EditTool, Drag, Tap):
             if not isinstance(renderer.glyph, (MultiLine, Patches)):
                 incompatible_renderers.append(renderer)
         if incompatible_renderers:
-            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            glyph_types = ', '.join(type(renderer.glyph).__name__ for renderer in incompatible_renderers)
+            return "%s glyph type(s) found." % glyph_types
+
+    @error(INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER)
+    def _check_compatible_vertex_renderer(self):
+        if self.vertex_renderer is None:
+            return
+        glyph = self.vertex_renderer.glyph
+        if not isinstance(glyph, XYGlyph):
+            return "glyph type %s found." % type(glyph).__name__
+
+class FreehandDrawTool(EditTool, Drag, Tap):
+    ''' *toolbar icon*: |freehand_draw_icon|
+
+    The FreehandDrawTool allows freehand drawing of ``Patches`` and
+    ``MultiLine`` glyphs. The glyph to draw may be defined via the
+    ``renderers`` property.
+
+    The tool will automatically modify the columns on the data source
+    corresponding to the ``xs`` and ``ys`` values of the glyph. Any
+    additional columns in the data source will be padded with the
+    declared ``empty_value``, when adding a new point.
+
+    The supported actions include:
+
+    * Draw vertices: Click and drag to draw a line
+
+    * Delete patch/multi-line: Tap a patch/multi-line to select it
+      then press <<backspace>> key while the mouse is within the plot
+      area.
+
+    .. |freehand_draw_icon| image:: /_images/icons/FreehandDraw.png
+        :height: 18pt
+    '''
+
+    num_objects = Int(default=0, help="""
+    Defines a limit on the number of patches or multi-lines that can
+    be drawn. By default there is no limit on the number of objects,
+    but if enabled the oldest drawn patch or multi-line will be
+    overwritten when the limit is reached.
+    """)
+
+    @error(INCOMPATIBLE_POLY_DRAW_RENDERER)
+    def _check_compatible_renderers(self):
+        incompatible_renderers = []
+        for renderer in self.renderers:
+            if not isinstance(renderer.glyph, (MultiLine, Patches)):
+                incompatible_renderers.append(renderer)
+        if incompatible_renderers:
+            glyph_types = ', '.join(type(renderer.glyph).__name__ for renderer in incompatible_renderers)
             return "%s glyph type(s) found." % glyph_types
 
 class PolyEditTool(EditTool, Drag, Tap):
@@ -1277,6 +1503,6 @@ class PolyEditTool(EditTool, Drag, Tap):
             if not isinstance(renderer.glyph, (MultiLine, Patches)):
                 incompatible_renderers.append(renderer)
         if incompatible_renderers:
-            glyph_types = ', '.join([type(renderer.glyph).__name__
-                                     for renderer in incompatible_renderers])
+            glyph_types = ', '.join(type(renderer.glyph).__name__
+                                    for renderer in incompatible_renderers)
             return "%s glyph type(s) found." % glyph_types
