@@ -1,5 +1,6 @@
 import logging
 
+import six
 import pytest
 from tornado.websocket import StreamClosedError, WebSocketClosedError
 
@@ -10,7 +11,9 @@ from bokeh.util.logconfig import basicConfig
 # needed for caplog tests to function
 basicConfig()
 
+@pytest.mark.skipif(six.PY2, reason="this test doesn't work on Python 2 due to 'fake self' hack.")
 @pytest.mark.parametrize('exc', [StreamClosedError, WebSocketClosedError])
+@pytest.mark.unit
 def test_send_message_raises(caplog, exc):
     class ExcMessage(object):
         def send(self, handler):
