@@ -19,7 +19,6 @@ from subprocess import Popen, PIPE
 from ..model import Model
 from ..settings import settings
 from .string import snakify
-from .deprecation import deprecated
 
 _plugin_umd = \
 """\
@@ -154,8 +153,13 @@ def _npmjs_path():
     global _npmjs
     if _npmjs is None:
         _npmjs = join(dirname(_nodejs_path()), "npm")
+        if sys.platform == "win32":
+            _npmjs += '.cmd'
     return _npmjs
 
+def _crlf_cr_2_lf(s):
+    return re.sub(r"\\r\\n|\\r|\\n", r"\\n", s)
+
 def _run(app, argv, input=None):
     proc = Popen([app] + argv, stdout=PIPE, stderr=PIPE, stdin=PIPE)
     (stdout, errout) = proc.communicate(input=None if input is None else json.dumps(input).encode())
@@ -163,7 +167,7 @@ def _run(app, argv, input=None):
     if proc.returncode != 0:
         raise RuntimeError(errout)
     else:
-        return stdout.decode('utf-8')
+        return _crlf_cr_2_lf(stdout.decode('utf-8'))
 
 def _run_nodejs(argv, input=None):
     return _run(_nodejs_path(), argv, input)
@@ -244,7 +248,7 @@ class TypeScript(Inline):
         .. code-block:: python
 
             class MyExt(Model):
-                __implementation__ = TypeScript(""" <TypeSctipt code> """)
+                __implementation__ = TypeScript(""" <TypeScript code> """)
 
     '''
     @property
@@ -252,14 +256,14 @@ class TypeScript(Inline):
         return "typescript"
 
 class JavaScript(Inline):
-    ''' An implementation for a Bokeh custom model in JavaSript
+    ''' An implementation for a Bokeh custom model in JavaScript
 
     Example:
 
         .. code-block:: python
 
             class MyExt(Model):
-                __implementation__ = Javacript(""" <Javactipt code> """)
+                __implementation__ = Javacript(""" <JavaScript code> """)
 
     '''
     @property
@@ -283,9 +287,6 @@ class FromFile(Implementation):
 
     '''
     def __init__(self, path):
-        if path.endswith(".tsx"):
-            deprecated((0, 12, 7), "TSX templates", "core/dom APIs")
-
         with io.open(path, encoding="utf-8") as f:
             self.code = f.read()
         self.file = path
@@ -294,7 +295,7 @@ class FromFile(Implementation):
     def lang(self):
         if self.file.endswith(".coffee"):
             return "coffeescript"
-        if self.file.endswith((".ts", ".tsx")):
+        if self.file.endswith(".ts"):
             return "typescript"
         if self.file.endswith(".js"):
             return "javascript"
@@ -302,7 +303,7 @@ class FromFile(Implementation):
             return "less"
 
 #: recognized extensions that can be compiled
-exts = (".coffee", ".ts", ".tsx", ".js", ".css", ".less")
+exts = (".coffee", ".ts", ".js", ".css", ".less")
 
 class CustomModel(object):
     ''' Represent a custom (user-defined) Bokeh model.
@@ -363,10 +364,26 @@ class CustomModel(object):
     def module(self):
         return "custom/%s" % snakify(self.full_name)
 
-def bundle_models(models):
-    """Create a bundle of `models`. """
-    custom_models = {}
+def _model_cache_no_op(model, implementation):
+    """Return cached compiled implementation"""
+    return None
 
+_CACHING_IMPLEMENTATION = _model_cache_no_op
+
+def get_cache_hook():
+    '''Returns the current cache hook used to look up the compiled
+       code given the CustomModel and Implementation'''
+    return _CACHING_IMPLEMENTATION
+
+def set_cache_hook(hook):
+    '''Sets a compiled model cache hook used to look up the compiled
+       code given the CustomModel and Implementation'''
+    global _CACHING_IMPLEMENTATION
+    _CACHING_IMPLEMENTATION = hook
+
+def _get_custom_models(models):
+    """Returns CustomModels for models with a custom `__implementation__`"""
+    custom_models = {}
     for cls in models:
         impl = getattr(cls, "__implementation__", None)
 
@@ -376,19 +393,11 @@ def bundle_models(models):
 
     if not custom_models:
         return None
+    return custom_models
 
+def _compile_models(custom_models):
+    """Returns the compiled implementation of supplied `models`. """
     ordered_models = sorted(custom_models.values(), key=lambda model: model.full_name)
-
-    exports = []
-    modules = []
-
-    def read_json(name):
-        with io.open(join(bokehjs_dir, "js", name + ".json"), encoding="utf-8") as f:
-            return json.loads(f.read())
-
-
-    bundles = ["bokeh", "bokeh-api", "bokeh-widgets", "bokeh-tables", "bokeh-gl"]
-    known_modules = set(sum([ read_json(name) for name in bundles ], []))
     custom_impls = {}
 
     dependencies = []
@@ -401,13 +410,35 @@ def bundle_models(models):
 
     for model in ordered_models:
         impl = model.implementation
-        compiled = nodejs_compile(impl.code, lang=impl.lang, file=impl.file)
+        compiled = _CACHING_IMPLEMENTATION(model, impl)
+        if compiled is None:
+            compiled = nodejs_compile(impl.code, lang=impl.lang, file=impl.file)
 
         if "error" in compiled:
             raise CompilationError(compiled.error)
 
         custom_impls[model.full_name] = compiled
 
+    return custom_impls
+
+def bundle_models(models):
+    """Create a bundle of `models`. """
+
+    custom_models = _get_custom_models(models)
+    if custom_models is None:
+        return
+
+    exports = []
+    modules = []
+
+    def read_json(name):
+        with io.open(join(bokehjs_dir, "js", name + ".json"), encoding="utf-8") as f:
+            return json.loads(f.read())
+
+    bundles = ["bokeh", "bokeh-api", "bokeh-widgets", "bokeh-tables", "bokeh-gl"]
+    known_modules = set(sum([ read_json(name) for name in bundles ], []))
+    custom_impls = _compile_models(custom_models)
+
     extra_modules = {}
 
     def resolve_modules(to_resolve, root):
@@ -456,7 +487,7 @@ def bundle_models(models):
         return resolved
 
     def resolve_deps(deps, root):
-        custom_modules = set([ model.module for model in custom_models.values() ])
+        custom_modules = set(model.module for model in custom_models.values())
         missing = set(deps) - known_modules - custom_modules
         return resolve_modules(missing, root)
 
@@ -479,8 +510,8 @@ def bundle_models(models):
 
     sep = ",\n"
 
-    exports = sep.join([ _export_template % dict(name=name, module=module) for (name, module) in exports ])
-    modules = sep.join([ _module_template % dict(module=module, source=code) for (module, code) in modules ])
+    exports = sep.join(_export_template % dict(name=name, module=module) for (name, module) in exports)
+    modules = sep.join(_module_template % dict(module=module, source=code) for (module, code) in modules)
 
     content = _plugin_template % dict(prelude=_plugin_prelude, exports=exports, modules=modules)
     return _plugin_umd % dict(content=content)
